﻿#region Using directives

using System;
using System.Diagnostics;
using System.Collections.Generic;
using System.Text;

#endregion

namespace Matrix4PerfTest
{
	class Program
	{
		#region Private utility functions
		private static Matrix4PerfTest.NotGeneric.Matrix4F CreateRandomNotGeneric(Random r)
		{
			return new Matrix4PerfTest.NotGeneric.Matrix4F(
				(float)r.NextDouble(), (float)r.NextDouble(), (float)r.NextDouble(), (float)r.NextDouble(),
				(float)r.NextDouble(), (float)r.NextDouble(), (float)r.NextDouble(), (float)r.NextDouble(),
				(float)r.NextDouble(), (float)r.NextDouble(), (float)r.NextDouble(), (float)r.NextDouble(),
				(float)r.NextDouble(), (float)r.NextDouble(), (float)r.NextDouble(), (float)r.NextDouble()
				);
		}
		private static Matrix4PerfTest.Generic.Matrix4<float> CreateRandomGeneric(Random r)
		{
			return new Matrix4PerfTest.Generic.Matrix4<float>(
				(float)r.NextDouble(), (float)r.NextDouble(), (float)r.NextDouble(), (float)r.NextDouble(),
				(float)r.NextDouble(), (float)r.NextDouble(), (float)r.NextDouble(), (float)r.NextDouble(),
				(float)r.NextDouble(), (float)r.NextDouble(), (float)r.NextDouble(), (float)r.NextDouble(),
				(float)r.NextDouble(), (float)r.NextDouble(), (float)r.NextDouble(), (float)r.NextDouble()
				);
		}
		private static Microsoft.DirectX.Matrix CreateRandomDX(Random r)
		{
			Microsoft.DirectX.Matrix result = new Microsoft.DirectX.Matrix();
			result.M11 = (float)r.NextDouble();
			result.M12 = (float)r.NextDouble();
			result.M13 = (float)r.NextDouble();
			result.M14 = (float)r.NextDouble();

			result.M21 = (float)r.NextDouble();
			result.M22 = (float)r.NextDouble();
			result.M23 = (float)r.NextDouble();
			result.M24 = (float)r.NextDouble();
			
			result.M31 = (float)r.NextDouble();
			result.M32 = (float)r.NextDouble();
			result.M33 = (float)r.NextDouble();
			result.M34 = (float)r.NextDouble();

			result.M41 = (float)r.NextDouble();
			result.M42 = (float)r.NextDouble();
			result.M43 = (float)r.NextDouble();
			result.M44 = (float)r.NextDouble();

			return result;
		}
		#endregion

		public const int IndicesSize = 10000;
		public const int MatrixBufferSize = 1000;
		public const int TestCount   = 10000000;

		private Random _rnd = new Random();
		private List<int> _indices = new List<int>(IndicesSize);
		private Matrix4PerfTest.Generic.Matrix4<float>[] _genMatrices = new Matrix4PerfTest.Generic.Matrix4<float>[MatrixBufferSize];
		private Matrix4PerfTest.NotGeneric.Matrix4F[] _ngenMatrices = new Matrix4PerfTest.NotGeneric.Matrix4F[MatrixBufferSize];
		private Microsoft.DirectX.Matrix[] _dxMatrices = new Microsoft.DirectX.Matrix[MatrixBufferSize];

		private void InitializeBuffers()
		{
			// Fill the matrix buffers with random matrices
			for (int i = 0; i < MatrixBufferSize; i++)
			{
				_genMatrices[i] = CreateRandomGeneric(_rnd);
				_ngenMatrices[i] = CreateRandomNotGeneric(_rnd);
				_dxMatrices[i] = CreateRandomDX(_rnd);
			}

			// Create random indices
			for (int i = 0; i < IndicesSize; i++)
			{
				_indices.Add(_rnd.Next(0, 1000));
			}
		}
		private void TestGeneric()
		{
			Console.Write("Testing generic 4x4 matrix implementation: ");

			Matrix4PerfTest.Generic.Matrix4<float>.Kernel = new Matrix4PerfTest.Generic.KernelMatrix4F();
			int currentIndex = 0;
			Stopwatch watch = new Stopwatch();

			watch.Start();
			for (int test = 0; test < TestCount; test++)
			{
				int ai = _indices[currentIndex];
				int bi = _indices[currentIndex++];

				Matrix4PerfTest.Generic.Matrix4<float> result =
					_genMatrices[ai] * _genMatrices[bi];

				// reached the end of index
				if (currentIndex == IndicesSize - 1)
					currentIndex = 0;
			}
			watch.Stop();

			Console.WriteLine(watch.Elapsed.ToString());
		}
		private void TestGenericDX()
		{
			Console.Write("Testing generic 4x4 matrix implementation (DirectX kernel): ");

			Matrix4PerfTest.Generic.Matrix4<float>.Kernel = new Matrix4PerfTest.Generic.KerneDirectXlMatrix4();
			int currentIndex = 0;
			Stopwatch watch = new Stopwatch();

			watch.Start();
			for (int test = 0; test < TestCount; test++)
			{
				int ai = _indices[currentIndex];
				int bi = _indices[currentIndex++];

				Matrix4PerfTest.Generic.Matrix4<float> result =
					_genMatrices[ai] * _genMatrices[bi];

				// reached the end of index
				if (currentIndex == IndicesSize - 1)
					currentIndex = 0;
			}
			watch.Stop();

			Console.WriteLine(watch.Elapsed.ToString());
		}
		private void TestNotGeneric()
		{
			Console.Write("Testing non-generic 4x4 matrix implementation: ");

			int currentIndex = 0;
			Stopwatch watch = new Stopwatch();

			watch.Start();
			for (int test = 0; test < TestCount; test++)
			{
				int ai = _indices[currentIndex];
				int bi = _indices[currentIndex++];

				Matrix4PerfTest.NotGeneric.Matrix4F result =
					_ngenMatrices[ai] * _ngenMatrices[bi];

				// reached the end of index
				if (currentIndex == IndicesSize - 1)
					currentIndex = 0;
			}
			watch.Stop();

			Console.WriteLine(watch.Elapsed.ToString());
		}
		private void TestDX()
		{
			Console.Write("Testing DirectX 4x4 matrix implementation: ");

			int currentIndex = 0;
			Stopwatch watch = new Stopwatch();

			watch.Start();
			for (int test = 0; test < TestCount; test++)
			{
				int ai = _indices[currentIndex];
				int bi = _indices[currentIndex++];

				Microsoft.DirectX.Matrix result =
					_dxMatrices[ai] * _dxMatrices[bi];

				// reached the end of index
				if (currentIndex == IndicesSize - 1)
					currentIndex = 0;
			}
			watch.Stop();

			Console.WriteLine(watch.Elapsed.ToString());
		}

		public Program()
		{
			InitializeBuffers();
		}

		public void RunTests()
		{
			TestGeneric();
			TestGenericDX();
			TestNotGeneric();
			TestDX();
		}

		static void Main(string[] args)
		{
			Program p = new Program();
			p.RunTests();
			Console.ReadKey();
		}
	}
}
